import sys
import mazegroup.imports as imports
imports.importMGPackages()
import mazegroup.commands as commands
import mazegroup.utils as utils

args = sys.argv[1:]

def main():
    if len(args) >= 1:
        command = args[0]
        command_args = args[1:]
        r = commands.executeCommand(command, command_args)
        if type(r) != utils.NoError:
            print(r.message)
    else:
        print("Usage : mazegroup <command> <args>")

if __name__ == "__main__":
    main()



import uuid
import random

class Void():
    def __new__(cls) -> None: pass

class NoError(): pass
class Error():
    def __init__(self, message:str="", id:int=0):
        self.message = message.capitalize()
        if len(self.message):
            if self.message[-1] == ".": self.message = self.message[:-1]
        self.message += "."; self.id = id

def RandIDv1():
    j = [*str(uuid.uuid1()).replace("-", "")]
    random.shuffle(j)
    for i in range(random.randint(1, 80)):
        j += hex(random.randint(1, 80)).replace("0x", "")
    random.shuffle(j)
    j = "".join(j)
    return j

def RandIDv1Tiny():
    j = [*str(uuid.uuid1()).replace("-", "")]
    random.shuffle(j)
    for i in range(random.randint(1, 80)):
        j += hex(random.randint(1, 80)).replace("0x", "")
    random.shuffle(j)
    j = "".join(j)
    return j[len(j) - 10:]


def importMGPackages():
    packages = ["echo", "pyexec", "pyeval", "help", "ls", "pypkg", "sc", "cd", "shell", "calc", "quit"]
    for package in packages:
        exec(f"import mazegroup.{package}", globals())



import mazegroup.commands as commands
import mazegroup.cli as cli
import os
import colorama
from colorama import Fore, Back

colorama.init()

class Command():
    def __init__(self) -> None:
        self.options_available = {
            "nocolor": ["-nc", "--nocolor"]
        }

        self.shell_mode = ""

        self.name = "shell"
        self.command_class = commands.Command(self.name, self.func, commands.Args([commands.Arg("options", required=False)], overflow=True), True)
        self.command_class.register()

    def optionOn(self, options:list, args:dict, option_type:str) -> bool:
        for option in options:
            arg = args[option]

            if arg.active_value in self.options_available[option_type]: return True
            return False

    def func(self, args:dict):
        print(Fore.LIGHTRED_EX + "MazeGroup.py " + Fore.BLACK + Back.LIGHTRED_EX + " Shell " + Fore.RESET + Back.RESET)
        print(Fore.WHITE + "For system shell commands, use before `§` the command.")
        print("For calc expressions, use before `%` the command." + Fore.RESET)
        while True:
            s = ""
            if self.shell_mode == "shell":
                s = "§ "
            elif self.shell_mode == "calc":
                s = "% "
            print(f"{Fore.LIGHTWHITE_EX}{Back.BLUE} MG {Back.LIGHTWHITE_EX} {Fore.BLACK}{os.getcwd()} {Back.RESET}{Fore.LIGHTBLUE_EX} > {s}{Fore.RESET}{Fore.LIGHTWHITE_EX}", end="")
            command = input().strip()
            cmd = command
            if self.shell_mode == "shell":
                command = "§ " + command
            elif self.shell_mode == "calc":
                command = "% " + command
            try:
                if len(cmd):
                    if command[0] == "§":
                        self.shell_mode = "shell"
                        if len(command[1:]):
                            os.system(command[1:])
                    elif command[0] == "%":
                        self.shell_mode = "calc"
                        if len(command[1:]):
                            args = command.strip().split()
                            if len(args) >= 1:
                                command_args_ = args
                                command_args = []
                                for arg in command_args_:
                                    command_args.append(arg.replace("\\s", " "))
                                r = cli.commands.executeCommand("calc", command_args)
                                if type(r) != cli.utils.NoError:
                                    print(f"{Fore.RED}{r.message}{Fore.RESET}")
                    else:
                        args = command.strip().split()
                        if len(args) >= 1:
                            command = args[0]
                            command_args_ = args[1:]
                            command_args = []
                            for arg in command_args_:
                                command_args.append(arg.replace("\\s", " "))
                            r = cli.commands.executeCommand(command, command_args)
                            if type(r) != cli.utils.NoError:
                                print(f"{Fore.RED}{r.message}{Fore.RESET}")
                        else:
                            print(f"{Fore.RED}Usage : <command> <args>{Fore.RESET}")
                else:
                    self.shell_mode = ""
            except Exception as e:
                print(f"{Fore.RED}Exception : {e}{Fore.RESET}")
Command()



import mazegroup.utils as utils
import pickle
import os
import copy

COMMANDS = {}

script_path = os.path.dirname(os.path.abspath(__file__))

def loadPickleVar():
    global COMMANDS
    pth = os.path.join(script_path, "saves", "commands.pkl")
    if os.path.exists(pth):
        COMMANDS = pickle.load(open(pth, "rb"))

def savePickleVar():
    pth = os.path.join(script_path, "saves", "commands.pkl")
    with open(pth, 'wb') as f:
        pickle.dump(COMMANDS, f)

def delPickleVar():
    global COMMANDS
    pth = os.path.join(script_path, "saves", "commands.pkl")
    if os.path.exists(pth):
        os.remove(pth)
    COMMANDS = {}

loadPickleVar()
savePickleVar()

class Arg():
    def __init__(self, name:str, default:any=None, required:bool=False) -> None:
        self.name = name
        self.default = default
        self.required = required

        self.active_value = None

    def parse(self, value:str):
        r = value
        py_eval_prefix = "py:"
        if value.lower().startswith(py_eval_prefix):
            try:
                r = eval(value[len(py_eval_prefix):])
            except Exception as e:
                return utils.Error(f"Python Eval Exception : {e}")
        return r

    def tryToSet(self, value:any=None):
        if not value and self.required:
            return utils.Error("The value is required")
        elif not value:
            self.active_value = self.parse(self.default)
            if type(self.active_value) == utils.Error:
                return self.active_value
        else:
            self.active_value = self.parse(value)
            if type(self.active_value) == utils.Error:
                return self.active_value
        return utils.NoError()
    
class Args():
    def __init__(self, args:list=[], overflow:bool=False) -> None:
        self.args_ = args
        self.args = {}
        for arg in self.args_:
            self.args[arg.name] = arg
        self.overflow = overflow

    def addArg(self, arg:Arg): self.args.append(arg)

    def tryArgs(self, args:list):
        ag = []
        for arg in self.args.values():
            if arg.required: ag.append(arg)
        if len(args) > len(self.args) and not self.overflow:
            return utils.Error(f"Argument(s) overflow, {len(args)} was given but {len(self.args)} was waited")
        elif len(args) < len(ag):
            return utils.Error(f"Missing argument(s), {len(args)} was given but {len(ag)} was waited")
        for arg_index, value in enumerate(args):
            if type(value) == dict:
                try:
                    arg_index = value.keys()[0]
                except:
                    return utils.Error()
            try:
                if type(arg_index) == str:
                    r = self.args[arg_index].tryToSet(value)
                else:
                    i = ""
                    for k, j in enumerate(self.args.keys()):
                        if k == arg_index:
                            i = j
                    r = self.args[i].tryToSet(value)
            except:
                id = utils.RandIDv1Tiny()
                arg = Arg(id)
                r = arg.tryToSet(value)
                self.args[id] = arg
            if type(r) != utils.NoError:
                return r
        return utils.NoError()
    
    def getArgsNames(self) -> list:
        r = []
        for arg in self.args.keys():
            r.append(arg)
        return r

class Command():
    def __init__(self, name:str=None, function=utils.Void, args:Args=Args(), unique:bool=True) -> None:
        self.function = copy.deepcopy(function)
        if not name:
            name = f'{function=}'.split('=')[0].strip().lower()
        self.name = name
        self.args = args
        self.unique = unique

    def register(self):
        #loadPickleVar()
        if self.name in COMMANDS.keys():
            return utils.Error(f"A command was already registered at the name '{self.name}'")
        self_copy = copy.deepcopy(self)
        COMMANDS[self.name] = {
            "function": self_copy.function,
            "args": self_copy.args,
            "class" : self_copy
        }
        savePickleVar()
        return utils.NoError()
    
    def execute(self, args:any):
        if type(args) == str:
            args = args.split()
        elif type(args) == list:
            pass
        elif type(args) == dict:
            args = args.keys()
        else:
            return utils.Error(f"Bad arguments type on the '{self.name}' command execution")
        r = self.args.tryArgs(args)
        if type(r) != utils.NoError:
            _ = self.args.getArgsNames(); __ = ""
            args_ = []
            for arg in self.args.args_:
                if arg.required: args_.append(arg)
            if not len(args_) < len(self.args.args):
                for arg in _: __ += f"<{arg}>"
            else:
                __ = f"<...>"
            return utils.Error(f"{r.message}\nUsage : mazegroup {self.name} {__}")
        try:
            if self.unique:
                self.function(self.args.args)
            else:
                self.function(*self.args.args)
        except Exception as e:
            return utils.Error(f"Exception during the execution of the command '{self.name}' : {e}")
        return utils.NoError()

def executeCommand(command:str, args:list):
    loadPickleVar()
    command = command.lower()
    if command in COMMANDS.keys():
        r = COMMANDS[command]["class"].execute(args)
        if type(r) != utils.NoError:
            return r
    else:
        return utils.Error("Command not found")
    return utils.NoError()



import mazegroup.commands as commands

import re
import random as rd

class Exp():
    def __init__(self) -> None:
        self.lexical = {
            "syn_operators": {
                "plus": ["+"],
                "minus": ["-"],
                "divide": ["/"],
                "mul": ["*"],
                "is_equal": ["==", "?="],
                "is_not_equal": ["!=", "!?="],
                "more_equal": [">="],
                "less_equal": ["<="],
                "more": [">"],
                "less": ["<"],
                "concat": ["&"],
                "power": ["^"],
                "bracket_open": ["("],
                "bracket_close": [")"]
            },
            "strings": {
                "string": ["\"", "'"],
                "backslash": ["\\\\"],
                "new_line": ["\\n"],
                "tab": ["\\t"],
                "db_quotes": ["\\\""],
                "quote": ["\\'"]
            }
        }

        self.numbers = [*"0123456789"]

        self.strings = {}
    
    def genStringID(self):
        r = "#"
        for _ in range(16):
            r += rd.choice(self.numbers)
        if r in self.strings.keys():
            r = self.genStringID()
        return r
    
    def mkStringReference(self, string:str, id:str):
        self.strings[id] = string

    def refAllStrings(self, exp:str):
        ls = self.lexical["strings"]
        for i in ls["backslash"]:
            exp = exp.replace(i, "\\")
        for k, h in {"new_line": "\n", "tab": "\t", "db_quotes": "š", "quote": "ž"}.items():
            for i in ls[k]:
                exp = exp.replace(i, h)
        string = False
        buffer = ""
        buffer2 = ""
        current_sep = ""
        strings = []
        for char in [*exp]:
            ex = char in ls["string"]
            if current_sep != "":
                ex = char == current_sep
            if ex:
                if string:
                    current_sep = ""
                    string = False
                    id = self.genStringID()
                    buffer = buffer.replace("š", "\"").replace("ž", "\'")
                    strings.append([buffer, id])
                    buffer = ""
                    buffer2 += id
                else:
                    string = True
                    current_sep = char
            else:
                if string:
                    buffer += char
                else:
                    buffer2 += char
        for string_ in strings:
            string = string_[0]
            id = string_[1]
            self.mkStringReference(string, id)
        return buffer2
    
    def getStringFromRefID(self, id:str):
        for string_id, string in self.strings.items():
            if id == string_id:
                return string

    def split_spec(self, exp:str):
        for k, ops in self.lexical["syn_operators"].items():
            for op in ops:
                exp = re.sub(r'(?<!\S)' + re.escape(op) + r'(?!\S)', ' ' + op + ' ', exp)

        exp = re.sub(r'(?<=\d|\))(?=[^\d\)])|(?<=[^\d\()])(?=\d|\()|(?<=\()(?=\d)|(?<=\d)(?=\))', ' ', exp)
        
        return " ".join(exp.split())
    
    def convertFloatToIntIfPossible(self, num):
        if isinstance(num, float):
            if num.is_integer():
                return int(num)
        return float(num)

    def parse(self, exp:str):
        exp = exp.strip()
        exp = self.refAllStrings(exp)
        tokens_ = self.split_spec(exp).split()
        string_id = None
        tokens = []
        for ind, token in enumerate(tokens_):
            if token == "#":
                try:
                    tokens.append(tokens_[ind] + tokens_[ind + 1])
                except:
                    tokens.append(token)
                tokens_.pop(ind + 1)
            else:
                tokens.append(token)
        parsed = []
        for token in tokens:
            type = "UNK"
            nb_type = ""
            op_type = ""
            if token[0] in self.numbers:
                type = "NB"
                nb_type = "INT"
                if "." in token:
                    nb_type = "DEC"
            if not type == "NB":
                if token[0] == "#":
                    token = self.getStringFromRefID(token)
                    type = "STR"
            for k, op in self.lexical["syn_operators"].items():
                for op_ in op:
                    if token == op_:
                        type = "OP"
                        op_type = k
            r = [token, type]
            if op_type != "":
                r.append(op_type)
            if nb_type != "":
                r.append(nb_type)
            parsed.append(r)
        parsed2 = []

        # Ici, utilise l'algorithme Shunting Yard pour convertir en notation postfixe
        precedence = {
            "mul": 2,
            "divide": 2,
            "power": 2,
            "is_equal": 2,
            "is_not_equal": 2,
            "more_equal": 2,
            "less_equal": 2,
            "more": 2,
            "less": 2,
            "concat": 2,
            "plus": 1,
            "minus": 1
        }

        stack = []
        for token in parsed:
            if token[1] == "NB":
                parsed2.append(token)
            elif token[1] == "STR":
                parsed2.append(token)
            elif token[1] == "OP":
                if token[2] == "bracket_open":
                    stack.append(token)
                elif token[2] == "bracket_close":
                    while stack and stack[-1][2] != "bracket_open":
                        parsed2.append(stack.pop())
                    if stack:
                        stack.pop()  # Dépiler la parenthèse ouvrante
                else:
                    while stack and precedence.get(stack[-1][2], 0) >= precedence.get(token[2], 0):
                        parsed2.append(stack.pop())
                    stack.append(token)
        while stack:
            parsed2.append(stack.pop())

        return parsed2

    def exec(self, parsed_exp):
        stack = []
        for token in parsed_exp:
            if token[1] == "NB":
                stack.append(self.convertFloatToIntIfPossible(float(token[0])))
            elif token[1] == "STR":
                stack.append(str(token[0]))
            elif token[1] == "OP":
                if token[2] == "plus":
                    try: b = stack.pop()
                    except: b = 1
                    try: a = stack.pop()
                    except: a = 1
                    try:
                        stack.append(a + b)
                    except:
                        stack.append(str(a) + str(b))
                elif token[2] == "minus":
                    try: b = stack.pop()
                    except: b = 1
                    try: a = stack.pop()
                    except: a = 1
                    stack.append(a - b)
                elif token[2] == "divide":
                    try: b = stack.pop()
                    except: b = 1
                    try: a = stack.pop()
                    except: a = 1
                    stack.append(a / b)
                elif token[2] == "mul":
                    try: b = stack.pop()
                    except: b = 1
                    try: a = stack.pop()
                    except: a = 1
                    stack.append(a * b)
                elif token[2] == "power":
                    try: b = stack.pop()
                    except: b = 1
                    try: a = stack.pop()
                    except: a = 1
                    stack.append(a ** b)
                elif token[2] == "is_equal":
                    try: b = stack.pop()
                    except: b = 1
                    try: a = stack.pop()
                    except: a = 1
                    stack.append(int(a == b))
                elif token[2] == "is_not_equal":
                    try: b = stack.pop()
                    except: b = 1
                    try: a = stack.pop()
                    except: a = 1
                    stack.append(int(a != b))
                elif token[2] == "more_equal":
                    try: b = stack.pop()
                    except: b = 1
                    try: a = stack.pop()
                    except: a = 1
                    stack.append(int(a >= b))
                elif token[2] == "less_equal":
                    try: b = stack.pop()
                    except: b = 1
                    try: a = stack.pop()
                    except: a = 1
                    stack.append(int(a <= b))
                elif token[2] == "more":
                    try: b = stack.pop()
                    except: b = 1
                    try: a = stack.pop()
                    except: a = 1
                    stack.append(int(a > b))
                elif token[2] == "less":
                    try: b = stack.pop()
                    except: b = 1
                    try: a = stack.pop()
                    except: a = 1
                    stack.append(int(a < b))
                elif token[2] == "concat":
                    b = stack.pop()
                    a = stack.pop()
                    if type(a) in [int, float] and type(b) in [int, float]:
                        stack.append(int(str(int(a)) + str(int(b))))
                    elif type(a) == str and type(b) == str:
                        stack.append(str(a) + str(b))
                    elif type(a) in [int, float]:
                        try:
                            stack.append(self.convertFloatToIntIfPossible(float(str(a) + str(b))))
                        except:
                            stack.append(str(str(a) + str(b)))
                    else:
                        stack.append(str(str(a) + str(b)))
        return stack[0] if stack else None

class echoCommand():
    def __init__(self) -> None:
        self.name = "calc"
        self.command_class = commands.Command(self.name, self.echo, commands.Args(overflow=True), True)
        self.command_class.register()

    def echo(self, args:dict):
        expression = ""
        for arg in args.values():
            expression += arg.active_value + " "
        parsed_exp = Exp().parse(expression)
        result = Exp().exec(parsed_exp)
        if result:
            print(result)
        
echoCommand()


"""
# MazeGroup.py
MazeGroup.py is an general prupose library for Python.
"""

import mazegroup.imports as imports
import os
#import mazegroup.commands as commands

#commands.delPickleVar()
#commands.savePickleVar()

script_path = os.path.dirname(os.path.abspath(__file__))
pth = os.path.join(script_path, "saves", "commands.pkl")
if os.path.exists(pth): os.remove(pth)

class Greet():
    def __init__(self) -> None: print("Hello! Welcome in MazeGroup.py")



import mazegroup.commands as commands

class echoCommand():
    def __init__(self) -> None:
        self.name = "quit"
        self.command_class = commands.Command(self.name, self.echo, commands.Args(overflow=True), True)
        self.command_class.register()
        self.command_class.name = "exit"
        self.command_class.register()

    def echo(self, args:dict):
        exit()

echoCommand()


import mazegroup.commands as commands

class Command():
    def __init__(self) -> None:
        self.name = "pyexec"
        self.command_class = commands.Command(self.name, self.func, commands.Args(overflow=True), True)
        self.command_class.register()

    def func(self, args:dict):
        expression = ""
        for arg in args.values():
            expression += arg.active_value + " "
        try:
            exec(expression)
        except Exception as e:
            print("Pyexec exception :", e)

Command()



import mazegroup.commands as commands

class echoCommand():
    def __init__(self) -> None:
        self.name = "echo"
        self.command_class = commands.Command(self.name, self.echo, commands.Args(overflow=True), True)
        self.command_class.register()

    def echo(self, args:dict):
        for arg in args.values():
            print(arg.active_value, end=" ")
        print()

echoCommand()



import mazegroup.commands as commands
import os

class Command():
    def __init__(self) -> None:
        self.name = "ls"
        self.command_class = commands.Command(self.name, self.func, commands.Args([commands.Arg("path", required=False)], overflow=False), True)
        self.command_class.register()

    def func(self, args:dict):
        if args[list(args.keys())[0]].active_value == None:
            path = os.getcwd()
        else:
            path = args[list(args.keys())[0]].active_value
        ls = "\n".join(os.listdir(path))
        print(f"Path : {path}\n{ls}")
            
Command()



import mazegroup.commands as commands

class Command():
    def __init__(self) -> None:
        self.name = "help"
        self.command_class = commands.Command(self.name, self.func, commands.Args(overflow=True), True)
        self.command_class.register()

    def func(self, args:dict):
        cmd_str = ""
        commands.loadPickleVar()
        commands_var = dict(sorted(commands.COMMANDS.items()))
        for name, command in commands_var.items():
            command = command["class"]
            _ = command.args.getArgsNames(); __ = ""
            if not len(command.args.args_) == 0:
                for arg in _: __ += f"<{arg}> "
            else:
                __ = f"<...>"
            cmd_str += f"\t{name} {__}\n"
        print(f"Commands available :\n{cmd_str}")

Command()



import mazegroup.commands as commands
import datetime
import os

class Command():
    def __init__(self) -> None:
        self.options_available = {
            "full": ["-f", "--full"]
        }

        self.version = "1"

        self.defaults = {
            "init": "\n".join(f"""
# __init__.py

\"\"\"
Package informations :
Name : <pkg_name>
Description : <pkg_description>
Author(s) : <pkg_author>
License : <pkg_license>
URL(s) : <pkg_url>
Creation date : <pkg_date>

Generated by MazeGroup.py Python Package Generator version {self.version}
MazeGroup Website : https://mazegroup.org/
MazeGroup.py GitHub : https://github.com/Geniusum/mazegroup.py
MazeGroup.py PyPi : https://pypi.org/project/mazegroup/
\"\"\"
""".splitlines()[1:]),
            "main": "\n".join(f"""
\"\"\"
Generated by MazeGroup.py Python Package Generator version {self.version}
MazeGroup Website : https://mazegroup.org/
MazeGroup.py GitHub : https://github.com/Geniusum/mazegroup.py
MazeGroup.py PyPi : https://pypi.org/project/mazegroup/
\"\"\"

import sys
import mazegroup as mg

class Main():
    def __init__(self, args:list=[]) -> None:
        self.args = args

    def script(self) -> any:
        pass # Your code here
        
if __name__ == "__main__":
    MainInstance = Main(sys.argv)
    MainInstance.script()
""".splitlines()[1:]),
            "package_json": "\n".join(f"""
{{
    "package": {{
        "name": "<pkg_name>",
        "description": "<pkg_description>",
        "author": "<pkg_author>",
        "license": "<pkg_license>",
        "url": "<pkg_url>",
        "date": "<pkg_date>"
    }},
    "from": "MazeGroup.py Python Package Generator version {self.version}"
}}
""".splitlines()[1:]),
            "package_xml": "\n".join(f"""
<data>
    <package>
        <name><pkg_name></name>
        <description><pkg_description></description>
        <author><pkg_author></author>
        <license><pkg_license></license>
        <url><pkg_url></url>
        <date><pkg_date></date>
    </package>
    <from>MazeGroup.py Python Package Generator version {self.version}</from>
</data>
""".splitlines()[1:]),
            "package_yml": "\n".join(f"""
package:
  name: "<pkg_name>"
  description: "<pkg_description>"
  author: "<pkg_author>"
  license: "<pkg_license>"
  url: "<pkg_url>"
  date: "<pkg_date>"
from: "MazeGroup.py Python Package Generator version {self.version}"
""".splitlines()[1:]),
            "package_csv": "\n".join(f"""
name,description,author,license,url,date,from
"<pkg_name>","<pkg_description>","<pkg_author>","<pkg_license>","<pkg_url>","<pkg_date>","MazeGroup.py Python Package Generator version {self.version}"
""".splitlines()[1:]),
            "package_toml": "\n".join(f"""
[package]
name = "<pkg_name>"
description = "<pkg_description>"
author = "<pkg_author>"
license = "<pkg_license>"
url = "<pkg_url>"
date = "<pkg_date>"

from = "MazeGroup.py Python Package Generator version {self.version}"
""".splitlines()[1:]),
            "package_ini": "\n".join(f"""
[package]
name = "<pkg_name>"
description = "<pkg_description>"
author = "<pkg_author>"
license = "<pkg_license>"
url = "<pkg_url>"
date = "<pkg_date>"

[from]
version = "MazeGroup.py Python Package Generator version {self.version}",
""".splitlines()[1:]),
            "package_py": "\n".join(f"""
package = {{
    "name": "<pkg_name>",
    "description": "<pkg_description>",
    "author": "<pkg_author>",
    "license": "<pkg_license>",
    "url": "<pkg_url>",
    "date": "<pkg_date>",
    "from": "MazeGroup.py Python Package Generator version {self.version}"
}}
""".splitlines()[1:])
        }

        self.tags = {
            "<pkg_name>": "",
            "<pkg_description>": "",
            "<pkg_author>": "",
            "<pkg_license>": "",
            "<pkg_url>": "",
            "<pkg_date>": ""
        }

        self.name = "pypkg"
        self.command_class = commands.Command(self.name, self.func, commands.Args([commands.Arg("name", required=True), commands.Arg("options", required=False)], overflow=True), True)
        self.command_class.register()
        
    def replaceTags(self, string:str) -> str:
        for key, value in self.tags.items(): string = string.replace(key, value)
        return string

    def optionOn(self, options:list, args:dict, option_type:str) -> bool:
        for option in options:
            arg = args[option]
            try:
                if str(arg.active_value).startswith("info:"):
                    infos = dict(eval(arg.active_value[5:]))
                    for tag in self.tags.keys():
                        try:
                            self.tags[tag] = infos[tag.replace("<pkg_", "").replace(">", "")]
                        except:
                            pass
            except: pass

        for option in options:
            arg = args[option]

            if arg.active_value in self.options_available[option_type]: return True
            return False

    def func(self, args:dict):
        options = list(args.keys())[1:]
        name = args["name"].active_value
        pkg_path = os.path.join(os.getcwd(), name)
        if not os.path.exists(pkg_path):
            try:
                self.tags["<pkg_name>"] = name
                self.tags["<pkg_date>"] = datetime.datetime.now().strftime("%d/%m/%Y")
                full = self.optionOn(options, args, "full")
                os.mkdir(pkg_path)
                init_file = open(os.path.join(pkg_path, "__init__.py"), "w")
                if full:
                    init_file.write(self.replaceTags(self.defaults["init"]))
                init_file.close()
                if full:
                    with open(os.path.join(pkg_path, "main.py"), "w") as f:
                        f.write(self.replaceTags(self.defaults["main"]))
                    with open(os.path.join(pkg_path, "package.json"), "w") as f:
                        f.write(self.replaceTags(self.defaults["package_json"]))
                    with open(os.path.join(pkg_path, "package.xml"), "w") as f:
                        f.write(self.replaceTags(self.defaults["package_xml"]))
                    with open(os.path.join(pkg_path, "package.yml"), "w") as f:
                        f.write(self.replaceTags(self.defaults["package_yml"]))
                    with open(os.path.join(pkg_path, "package.csv"), "w") as f:
                        f.write(self.replaceTags(self.defaults["package_csv"]))
                    with open(os.path.join(pkg_path, "package.toml"), "w") as f:
                        f.write(self.replaceTags(self.defaults["package_toml"]))
                    with open(os.path.join(pkg_path, "package.ini"), "w") as f:
                        f.write(self.replaceTags(self.defaults["package_ini"]))
                    with open(os.path.join(pkg_path, "package.py"), "w") as f:
                        f.write(self.replaceTags(self.defaults["package_py"]))
            except Exception as e:
                print(f"Error during the package generation : {e}")
        else:
            print("The directory already exists.")
            
Command()



import mazegroup.commands as commands
import os

class Command():
    def __init__(self) -> None:
        self.name = "cd"
        self.command_class = commands.Command(self.name, self.func, commands.Args([commands.Arg("path", required=True)], overflow=False), True)
        self.command_class.register()

    def func(self, args:dict):
        path = str(os.path.abspath(args[list(args.keys())[0]].active_value)).replace("\\", "/")
        os.chdir(path)
            
Command()


import mazegroup.commands as commands
import os
import sys
import shutil
from cryptography.fernet import Fernet
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC
import base64

class MCP():
    def __init__(self) -> None:
        self.script_path = os.path.dirname(os.path.abspath(__file__))
        self.temp_path = os.path.join(self.script_path, "temp")
        self.out_path = os.path.join(self.script_path, "out")
        self.ExistsDirOrMake(self.temp_path)
        self.ExistsDirOrMake(self.out_path)

    def ExistsDirOrMake(self, path):
        if not os.path.exists(path): os.mkdir(path)

    def Encrypt(self, source:str, key:str, salt):
        key = bytes(key, "utf-8")
        kdf = PBKDF2HMAC(
            algorithm=hashes.SHA256(),
            length=32,
            salt=salt,
            iterations=480000,
        )
        key = base64.urlsafe_b64encode(kdf.derive(key))
        cipher_suite = Fernet(key)
        return cipher_suite.encrypt(bytes(str(source), "utf-8")), salt

    def Decrypt(self, source:str, key:str, salt):
        if type(source) == bytes:
            source = source.decode("utf-8")
        if source.startswith("b'"):
            source = source[2:-1]
        key = bytes(key, "utf-8")
        kdf = PBKDF2HMAC(
            algorithm=hashes.SHA256(),
            length=32,
            salt=salt,
            iterations=480000,
        )
        key = base64.urlsafe_b64encode(kdf.derive(key))
        cipher_suite = Fernet(key)
        return str(cipher_suite.decrypt(source))

    def SecureCompressProcess(self, path:str, output:str=None, password:str="", increment:int=0, tar:str=None):
        if not len(password):
            password = "KEY"
        if not tar:
            tar = bytes(password, "utf-8")
        if type(tar) == str:
            tar = bytes(tar, "utf-8")
        if increment <= 0:
            increment = len(password)
        data = []
        salt = os.urandom(16)
        name = os.path.splitext(path)[0]
        if not output:
            output = self.out_path
        msc = os.path.join(output, name + ".msc")
        if os.path.exists(msc):
                os.remove(msc)
        if not os.path.exists(path):
            exit("This path don't exists.")
        if os.path.isfile(path):
            if os.path.exists(os.path.join(self.temp_path, name)):
                os.remove(os.path.join(self.temp_path, name))
            os.mkdir(os.path.join(self.temp_path, name))
            shutil.copy(path, os.path.join(self.temp_path, name))
            path = os.path.join(self.temp_path, name)
        for root, dirs, files in os.walk(path):
            for file in files:
                file_path = os.path.join(root, file)
                data.append([file_path, open(os.path.abspath(file_path)).read()])
            for dir in dirs:
                dir_path = os.path.join(root, dir)
                data.append([dir_path])
        encrypted = self.Encrypt(str(data), password, salt)
        for _ in range(increment - 1):
            encrypted = self.Encrypt(encrypted[0].decode("utf-8"), password, salt)
        content = {
            "salt": salt,
            "data": encrypted[0]
        }
        content_encrypted = self.Encrypt(str(content), password, tar)[0]
        f = open(msc, "w+")
        f.write(str(content_encrypted))
        f.close()
        print(f"Secure compress : the path is '{msc}'")
        #return content_encrypted
    
    def SCP_Decompress(self, path:str, output:str=None, password:str="", increment:int=0, tar:str=None):
        if not len(password):
            password = "KEY"
        if not tar:
            tar = bytes(password, "utf-8")
        if type(tar) == str:
            tar = bytes(tar, "utf-8")
        if increment <= 0:
            increment = len(password)
        
        if not output:
            output = self.out_path
        
        msc_file = os.path.splitext(path)[0] + ".msc"
        if not os.path.exists(msc_file):
            exit("The specified .msc file does not exist.")

        output = os.path.join(output, os.path.splitext(os.path.basename(msc_file))[0])
        
        with open(msc_file, "r") as f:
            content_encrypted = f.read().replace("b'", "").replace("'", "")
        
        content = self.Decrypt(content_encrypted, password, tar)
        content = eval(content)
        content = content.decode("utf-8")
        content = dict(eval(content))  # Convert string representation of dictionary to dictionary
        
        salt = content["salt"]
        data_encrypted = content["data"]
        
        for _ in range(increment):
            data_encrypted = self.Decrypt(data_encrypted, password, salt)

        data_encrypted = bytes(eval(data_encrypted))

        data = eval(data_encrypted)  # Convert string representation of data to list

        if not os.path.exists(output):
            os.mkdir(output)
        
        for item in data:
            if len(item) == 2:
                file_path, file_content = item
                file_content = file_content
                with open(os.path.join(os.path.dirname(output), file_path), "w") as f:
                    f.write(file_content)
            elif len(item) == 1:
                dir_path = item[0]
                os.makedirs(os.path.join(os.path.dirname(output), dir_path), exist_ok=True)

        print(f"Secure compress : the path is '{os.path.dirname(output)}'")

        #shutil.move(rc_temp, output)  # Move the decrypted files to the specified output directory

class Command():
    def __init__(self) -> None:
        self.name = "sc"
        self.command_class = commands.Command(self.name, self.func, commands.Args(overflow=True), True)
        self.command_class.register()

    def func(self, args_:dict):
        args = []
        for arg in args_.values():
            args.append(arg.active_value)
        if not args:
            print("Usage : mazegroup sc <compress/decompress> <path> <!* output path> <!* password> <!* increments> <!* tar>")
        else:
            if len(args) >= 2:
                if args[0] == "compress":
                    path = args[1]
                    try: output = args[2]
                    except: output = None
                    try: password = args[3]
                    except: password = "KEY"
                    try: increments = args[4]
                    except: increments = len(password)
                    try: tar = args[5]
                    except: tar = bytes(password, "utf-8")
                    ins = MCP()
                    ins.SecureCompressProcess(path, output, password, increments, tar)
                elif args[0] == "decompress":
                    path = args[1]
                    try: output = args[2]
                    except: output = None
                    try: password = args[3]
                    except: password = "KEY"
                    try: increments = args[4]
                    except: increments = len(password)
                    try: tar = args[5]
                    except: tar = bytes(password, "utf-8")
                    ins = MCP()
                    ins.SCP_Decompress(path, output, password, increments, tar)
                else:
                    print("Usage : mazegroup sc <compress/decompress> <path> <!* output path> <!* password> <!* increments> <!* tar>")
            else:
                print("Usage : mazegroup sc <compress/decompress> <path> <!* output path> <!* password> <!* increments> <!* tar>")
            
Command()
